/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.ant;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.apache.tools.ant.BuildException;

/**
 * This class is used to gathers the operation on the components
 * 
 * @author ddesjardins chamerling - eBMWebsourcing
 */
public abstract class AbstractComponentAntTask extends AbstractJBIAntTask {

    /**
     * Action : shutdown the component
     */
    protected static final String SHUTDOWN = "shutDown";

    /**
     * Action : start the component
     */
    protected static final String START = "start";

    /**
     * Action : stop the component
     */
    protected static final String STOP = "stop";

    /**
     * Action : Uninstall the component
     */
    protected static final String UNINSTALL = "uninstall";

    /**
     * Name of the JBI component in the JBI environment
     */
    protected String name;

    /**
     * The action to perform
     */
    private String action;

    /**
     * Creates a new instance of {@link AbstractComponentAntTask}
     * 
     * @param action
     */
    public AbstractComponentAntTask(String action) {
        this.action = action;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.ant.managers.AbstractJBIAntTask#doTask()
     */
    public void doTask() throws Exception {
        if (name == null) {
            throw new BuildException("Missing attribute 'name'");
        }

        if (UNINSTALL.equalsIgnoreCase(action)) {
            performUninstall();
        } else {
            performAction();
        }
    }

    /**
     * Perform an action on the component
     * 
     * @param action
     *            action to perform
     */
    protected void performAction() throws Exception {
        MBeanServerConnection connection = connector.getMBeanServerConnection();

        Object[] parameters = new Object[0];
        String[] signature = new String[0];
        ObjectName objectName = JBIJMXConnectorUtil.getComponentMBeanName(
            connection, name);

        if (objectName == null) {
            throw new BuildException("The component '" + name
                + "' is not installed");
        }

        try {
            log("Invoking '" + action + "' on component '" + name + "'");
        } catch (NullPointerException e) {
            // Exception thrown when execute outisde of the ant context
        }

        connection.invoke(objectName, action, parameters, signature);
    }

    /**
     * Uninstall a component
     * 
     * @param connector
     * @throws Exception
     */
    protected void performUninstall() throws Exception {
        MBeanServerConnection connection = connector.getMBeanServerConnection();

        Object[] parameters1 = new Object[1];
        Object[] parameters2 = new Object[0];
        Object[] parameters3 = new Object[2];
        parameters1[0] = name;
        parameters3[0] = name;
        parameters3[1] = true;
        String[] signature1 = new String[] {"java.lang.String"};
        String[] signature2 = new String[] {};
        String[] signature3 = new String[] {"java.lang.String", "boolean"};

        boolean result = false;
        try {
            // First call uninstall on installer
            ObjectName instName = (ObjectName) connection.invoke(
                JBIJMXConnectorUtil.getAdminServiceMBeanName(connection),
                "getInstallerByName", parameters1, signature1);
            connection.invoke(instName, "uninstall", parameters2, signature2);

            // Unload the installer
            result = ((Boolean) connection.invoke(JBIJMXConnectorUtil
                .getInstallationServiceMBeanName(connection),
                "unloadInstaller", parameters3, signature3)).booleanValue();
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (!result) {
            throw new BuildException("The component '" + name
                + "' has not been uninstalled");
        }

        log("The component '" + name + "' has been uninstalled");
    }

    /**
     * 
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }

}
